@@ -1,4 +1,5 @@
 import copy
+import decimal
 import math
 import pprint
 
@@ -6,14 +7,17 @@ import pytest
 
 from pint import Context, DimensionalityError, UnitRegistry, get_application_registry
 from pint.compat import np
+from pint.facets.plain.unit import UnitsContainer
+from pint.testing import assert_equal
 from pint.testsuite import QuantityTestCase, helpers
-from pint.unit import UnitsContainer
 from pint.util import ParserHelper
 
 
+from .helpers import internal
+
+
 # TODO: do not subclass from QuantityTestCase
 class TestIssues(QuantityTestCase):
-
     kwargs = dict(autoconvert_offset_to_baseunit=False)
 
     @pytest.mark.xfail
@@ -248,7 +252,6 @@ class TestIssues(QuantityTestCase):
         assert dis.value == acc.value * tim.value**2 / 2
 
     def test_issue85(self, module_registry):
-
         T = 4.0 * module_registry.kelvin
         m = 1.0 * module_registry.amu
         va = 2.0 * module_registry.k * T / m
@@ -261,7 +264,6 @@ class TestIssues(QuantityTestCase):
         helpers.assert_quantity_almost_equal(va.to_base_units(), vb.to_base_units())
 
     def test_issue86(self, module_registry):
-
         module_registry.autoconvert_offset_to_baseunit = True
 
         def parts(q):
@@ -333,7 +335,6 @@ class TestIssues(QuantityTestCase):
         helpers.assert_quantity_almost_equal(z, 5.1 * module_registry.meter)
 
     def test_issue104(self, module_registry):
-
         x = [
             module_registry("1 meter"),
             module_registry("1 meter"),
@@ -360,7 +361,6 @@ class TestIssues(QuantityTestCase):
         helpers.assert_quantity_almost_equal(y[0], module_registry.Quantity(1, "meter"))
 
     def test_issue105(self, module_registry):
-
         func = module_registry.parse_unit_name
         val = list(func("meter"))
         assert list(func("METER")) == []
@@ -395,7 +395,10 @@ class TestIssues(QuantityTestCase):
     def test_alternative_angstrom_definition(self, module_registry):
         module_registry.Quantity(2, "\u212B")
 
-    def test_micro_creation(self, module_registry):
+    def test_micro_creation_U03bc(self, module_registry):
+        module_registry.Quantity(2, "μm")
+
+    def test_micro_creation_U00b5(self, module_registry):
         module_registry.Quantity(2, "µm")
 
     @helpers.requires_numpy
@@ -445,10 +448,10 @@ class TestIssues(QuantityTestCase):
 
     def test_issue354_356_370(self, module_registry):
         assert (
-            "{:~}".format(1 * module_registry.second / module_registry.millisecond)
+            f"{1 * module_registry.second / module_registry.millisecond:~}"
             == "1.0 s / ms"
         )
-        assert "{:~}".format(1 * module_registry.count) == "1 count"
+        assert f"{1 * module_registry.count:~}" == "1 count"
         assert "{:~}".format(1 * module_registry("MiB")) == "1 MiB"
 
     def test_issue468(self, module_registry):
@@ -469,7 +472,6 @@ class TestIssues(QuantityTestCase):
 
     @helpers.requires_numpy
     def test_issue483(self, module_registry):
-
         a = np.asarray([1, 2, 3])
         q = [1, 2, 3] * module_registry.dimensionless
         p = (q**q).m
@@ -720,7 +722,7 @@ class TestIssues(QuantityTestCase):
 
     def test_issue1058(self, module_registry):
         """verify that auto-reducing quantities with three or more units
-        of same base type succeeds"""
+        of same plain type succeeds"""
         q = 1 * module_registry.mg / module_registry.g / module_registry.kg
         q.ito_reduced_units()
         assert isinstance(q, module_registry.Quantity)
@@ -728,7 +730,7 @@ class TestIssues(QuantityTestCase):
     def test_issue1062_issue1097(self):
         # Must not be used by any other tests
         ureg = UnitRegistry()
-        assert "nanometer" not in ureg._units
+        assert "nanometer" not in internal(ureg)._units
         for i in range(5):
             ctx = Context.from_lines(["@context _", "cal = 4 J"])
             with ureg.context("sp", ctx):
@@ -852,6 +854,29 @@ class TestIssues(QuantityTestCase):
             np.array((0.04, 0.09)),
         )
 
+    def test_issue1277(self, module_registry):
+        ureg = module_registry
+        assert ureg("%") == ureg("percent")
+        assert ureg("%") == ureg.percent
+        assert ureg("ppm") == ureg.ppm
+
+        a = ureg.Quantity("10 %")
+        b = ureg.Quantity("100 ppm")
+        c = ureg.Quantity("0.5")
+
+        assert f"{a}" == "10 percent"
+        assert f"{a:~}" == "10 %"
+        assert f"{b}" == "100 ppm"
+        assert f"{b:~}" == "100 ppm"
+
+        assert_equal(a, 0.1)
+        assert_equal(1000 * b, a)
+        assert_equal(c, 5 * a)
+
+        assert_equal((1 * ureg.meter) / (1 * ureg.kilometer), 0.1 * ureg.percent)
+        assert c.to("percent").m == 50
+        # assert c.to("%").m == 50  # TODO: fails.
+
     @helpers.requires_uncertainties()
     def test_issue_1300(self):
         module_registry = UnitRegistry()
@@ -859,6 +884,66 @@ class TestIssues(QuantityTestCase):
         m = module_registry.Measurement(1, 0.1, "meter")
         assert m.default_format == "~P"
 
+    @helpers.requires_babel()
+    def test_issue_1400(self, sess_registry):
+        q1 = 3 * sess_registry.W
+        q2 = 3 * sess_registry.W / sess_registry.cm
+        assert q1.format_babel("~", locale="es_Ar") == "3 W"
+        assert q1.format_babel("", locale="es_Ar") == "3 vatios"
+        assert q2.format_babel("~", locale="es_Ar") == "3.0 W / cm"
+        assert q2.format_babel("", locale="es_Ar") == "3.0 vatios por centímetros"
+
+    @helpers.requires_uncertainties()
+    def test_issue1611(self, module_registry):
+        from numpy.testing import assert_almost_equal
+        from uncertainties import ufloat
+
+        from pint import pint_eval
+
+        pint_eval.tokenizer = pint_eval.uncertainty_tokenizer
+
+        u1 = ufloat(1.2, 0.34)
+        u2 = ufloat(5.6, 0.78)
+        q1_u = module_registry.Quantity(u2 - u1, "m")
+        q1_str = str(q1_u)
+        q1_str = "{:.4uS}".format(q1_u)
+        q1_m = q1_u.magnitude
+        q2_u = module_registry.Quantity(q1_str)
+        # Not equal because the uncertainties are differently random!
+        assert q1_u != q2_u
+        q2_m = q2_u.magnitude
+
+        assert_almost_equal(q2_m.nominal_value, q1_m.nominal_value, decimal=9)
+        assert_almost_equal(q2_m.std_dev, q1_m.std_dev, decimal=4)
+
+        q3_str = "12.34(5678)e-066 m"
+        q3_u = module_registry.Quantity(q3_str)
+        q3_m = q3_u.magnitude
+        assert q3_m < 1
+
+    @helpers.requires_uncertainties
+    def test_issue1614(self, module_registry):
+        from uncertainties import UFloat, ufloat
+
+        q = module_registry.Quantity(1.0, "m")
+        assert isinstance(q, module_registry.Quantity)
+        m = module_registry.Measurement(2.0, 0.3, "m")
+        assert isinstance(m, module_registry.Measurement)
+
+        u1 = ufloat(1.2, 3.4)
+        u2 = ufloat(5.6, 7.8)
+        q1_u = module_registry.Quantity(u1, "m")
+        m1 = module_registry.Measurement(q1_u)
+        assert m1.value.magnitude == u1.nominal_value
+        assert m1.error.magnitude == u1.std_dev
+        m2 = module_registry.Measurement(5.6, 7.8)  # dimensionless
+        q2_u = module_registry.Quantity(m2)
+        assert isinstance(q2_u.magnitude, UFloat)
+        assert q2_u.magnitude.nominal_value == m2.value
+        assert q2_u.magnitude.nominal_value == u2.nominal_value
+        assert q2_u.magnitude.std_dev == m2.error
+        assert q2_u.magnitude.std_dev == u2.std_dev
+
 
 if np is not None:
 
@@ -909,7 +994,7 @@ def test_issue1498(tmp_path):
     def1 = tmp_path / "def1.txt"
     def2 = tmp_path / "def2.txt"
 
-    # A file that defines a new base unit and uses it in a context
+    # A file that defines a new plain unit and uses it in a context
     def0.write_text(
         """
     foo = [FOO]
@@ -920,12 +1005,12 @@ def test_issue1498(tmp_path):
     """
     )
 
-    # A file that defines a new base unit, then imports another file…
+    # A file that defines a new plain unit, then imports another file…
     def1.write_text(
         f"""
     foo = [FOO]
 
-    @import {str(def2)}
+    @import {def2.name}
     """
     )
 
@@ -938,9 +1023,8 @@ def test_issue1498(tmp_path):
     """
     )
 
-    # Succeeds with pint 0.18; fails with pint 0.19
     ureg1 = UnitRegistry()
-    ureg1.load_definitions(def1)  # ← FAILS
+    ureg1.load_definitions(def1)
 
     assert 12.0 == ureg1("1.2 foo").to("kg", "BAR").magnitude
 
@@ -953,7 +1037,7 @@ def test_issue1498b(tmp_path):
     def1_2 = tmp_path / "dir_a" / "def1_2.txt"
     def2 = tmp_path / "def2.txt"
 
-    # A file that defines a new base unit and uses it in a context
+    # A file that defines a new plain unit and uses it in a context
     def0.write_text(
         """
     foo = [FOO]
@@ -967,7 +1051,7 @@ def test_issue1498b(tmp_path):
     """
     )
 
-    # A file that defines a new base unit, then imports another file…
+    # A file that defines a new plain unit, then imports another file…
     def1.parent.mkdir()
     def1.write_text(
         """
@@ -1006,3 +1090,66 @@ def test_issue1498b(tmp_path):
     ureg1.load_definitions(def0)  # ← FAILS
 
     assert 12.0 == ureg1("1.2 foo").to("kg", "BAR").magnitude
+
+
+def test_backcompat_speed_velocity(func_registry):
+    get = func_registry.get_dimensionality
+    assert get("[velocity]") == UnitsContainer({"[length]": 1, "[time]": -1})
+    assert get("[speed]") == UnitsContainer({"[length]": 1, "[time]": -1})
+
+
+def test_issue1433(func_registry):
+    assert func_registry.Quantity("1 micron") == func_registry.Quantity("1 micrometer")
+
+
+def test_issue1527():
+    ureg = UnitRegistry(non_int_type=decimal.Decimal)
+    x = ureg.parse_expression("2 microliter milligram/liter")
+    assert x.magnitude.as_tuple()[1] == (2,)
+    assert x.to_compact().as_tuple()[1] == (2,)
+    assert x.to_base_units().as_tuple()[1] == (2,)
+    assert x.to("ng").as_tuple()[1] == (2,)
+
+
+def test_issue1621():
+    ureg = UnitRegistry(non_int_type=decimal.Decimal)
+    digits = ureg.Quantity("5.0 mV/m").to_base_units().magnitude.as_tuple()[1]
+    assert digits == (5, 0)
+
+
+def test_issue1631():
+    import pint
+
+    # Test registry subclassing
+    class MyRegistry(pint.UnitRegistry):
+        pass
+
+    assert MyRegistry.Quantity is pint.UnitRegistry.Quantity
+    assert MyRegistry.Unit is pint.UnitRegistry.Unit
+
+    ureg = MyRegistry()
+
+    u = ureg.meter
+    assert isinstance(u, ureg.Unit)
+    assert isinstance(u, pint.Unit)
+
+    q = 2 * ureg.meter
+    assert isinstance(q, ureg.Quantity)
+    assert isinstance(q, pint.Quantity)
+
+
+def test_issue1725(registry_empty):
+    registry_empty.define("dollar = [currency]")
+    assert registry_empty.get_compatible_units("dollar") == set()
+
+
+def test_issues_1505():
+    ur = UnitRegistry(non_int_type=decimal.Decimal)
+
+    assert isinstance(ur.Quantity("1m/s").magnitude, decimal.Decimal)
+    assert not isinstance(
+        ur.Quantity("m/s").magnitude, float
+    )  # unexpected success (magnitude should not be a float)
+    assert isinstance(
+        ur.Quantity("m/s").magnitude, decimal.Decimal
+    )  # unexpected fail (magnitude should be a decimal)
